
/*!
 *  @file     M95M02.c
 *  @author   Clint Stevenson (Casco Logix)
 *  @license  
 *
 *  The M95M02 is a SPI-interface EEPROM chip with 2Mbit capacity
 *
 *  This is the source file for the M95M02 Arduino library
 *
 *  @section  HISTORY
 *  
 *  v1.0 - First release
 *  v1.1 - Bitbucket Issue #1:
 *		   Fixed data types in read and write to unsigned int to support full page
 *         read/writes as well as reading/writing the entire chip. 
 *		   Bitbucket Issue #2, Issue #3:
 *		   Added proposed code for accessing the EEPROM's ID page.
 *  v1.2 - Bitbucket issue #4:
 *		   Fixed addressing bug where bitmask operation was using logical
 *         'AND' operator (&&) instead of bitwise 'AND' operator (&).
 */


#include "M95M02.h"
#include "multiplexer.h"
#include "spi.h"

#define M95M02_CMD_WREN								0x06
#define M95M02_CMD_WRDI								0x04
#define M95M02_CMD_RDSR								0x05
#define M95M02_CMD_WRSR								0x01
#define M95M02_CMD_READ								0x03
#define M95M02_CMD_WRITE							0x02
#define M95M02_CMD_READ_ID							0x83
#define M95M02_CMD_WRITE_ID							0x82
#define M95M02_CMD_READ_LOCK_STATUS					0x83
#define M95M02_CMD_LOCK_ID							0x82

#define M95M02_STATUS_REGISTER_SRWD_BIT				0x80
#define M95M02_STATUS_REGISTER_BP1_BIT				0x08
#define M95M02_STATUS_REGISTER_BP0_BIT				0x04
#define M95M02_STATUS_REGISTER_BP_MASK		(M95M02_STATUS_REGISTER_BP1_BIT | \
											M95M02_STATUS_REGISTER_BP0_BIT)
#define M95M02_STATUS_REGISTER_WEL_BIT				0x02
#define M95M02_STATUS_REGISTER_WIP_BIT				0x01

#define M95M02_SEND_ADDRESS_STATE_DONE 				0
#define M95M02_SEND_ADDRESS_STATE_BUSY 				1


void commandWREN (void);
void commandWRITE (void);
void commandREAD (void);
unsigned char M95M02_getWIP (void);
void M95M02_commandWREN (void);

/*!
 *  @function		write
 *  @parameters		address, data
 *	@returns		number of bytes written (always 1)
 *	@description	Writes one byte to the specified address.
*/
unsigned char M95M02_write (unsigned int address, unsigned char data)
{

	M95M02_commandWREN();
	
	EEPROM_CS;
	//M95M02_commandWREN();
	commandWRITE();										// Send WRITE command
	
    SendAndRcvByte((address& 0xFF0000) >> 16);	// MSB
    SendAndRcvByte((address& 0xFF00) >> 8);	
    SendAndRcvByte(address & 0xFF); 			// LSB
	
    SendAndRcvByte(data);
	 
	MULTI_CS_HIGH;
	//while (M95M02_getWIP() == 1); /* Write in progress */
	return 1;
}


/******************************************************************************/
/*!
 *  @function		write
 *  @parameters		address, pBuffer, numBytes
 *	@returns		number of bytes written
 *	@description	Writes the specified number of bytes starting at the 
 *					specified address. Data is written from a buffer which must
 *					be passed to the function. 
*/
 unsigned char M95M02_write_buffer (unsigned int address, unsigned char * pBuffer, unsigned int numBytes) 
{unsigned int index;
	EEPROM_CS;
	commandWRITE();										// Send WRITE command
	
    SendAndRcvByte((unsigned char)((address >> 16) & 0xFF));	// MSB
    SendAndRcvByte((unsigned char)((address >> 8) & 0xFF));	
    SendAndRcvByte((unsigned char)(address & 0xFF)); 			// LSB
    
	
    for (index = 0; index < numBytes; index++)
    {
		SendAndRcvByte(pBuffer[index]);
		// TODO: add loop exit to handle write errors
	}
	MULTI_CS_HIGH;
	return index;
}
  

/******************************************************************************/
/*!
 *  @function		read
 *  @parameters		address, pBuffer
 *	@returns		number of bytes read (always 1)
 *	@description	Reads one byte from the specified address
*/
unsigned char M95M02_read (unsigned int address)
{unsigned char l;
	
	M95M02_commandWREN();
	
	EEPROM_CS;
	commandREAD();										// Send READ command

    SendAndRcvByte((address& 0xFF0000) >> 16);	// MSB
    SendAndRcvByte((address& 0xFF00) >> 8);	
    SendAndRcvByte(address & 0xFF); 			// LSB
	
	l = SendAndRcvByte(0);
	MULTI_CS_HIGH;
    return l;
}


/******************************************************************************/
/*!
 *  @function		read
 *  @parameters		address, pBuffer, numBytes
 *	@returns		number of bytes read
 *	@description	Reads the specified number of bytes starting from the 
 *					specified address. Data is placed into a buffer which must
 *					be passed to the function. Buffer size must be greater than
 *					or equal to the number of bytes being read.
*/
unsigned char M95M02_read_buffer (unsigned int address, unsigned char * pBuffer, unsigned int numBytes)
{unsigned int index;
	EEPROM_CS;
	commandREAD();										// Send READ command

    SendAndRcvByte((unsigned char)((address >> 16) & 0xFF));	// MSB
    SendAndRcvByte((unsigned char)((address >> 8) & 0xFF));	
    SendAndRcvByte((unsigned char)(address & 0xFF)); 			// LSB
	
    
    for (index = 0; index < numBytes; index++ )
	{
		pBuffer[index] = SendAndRcvByte(0);
	}
	MULTI_CS_HIGH;
	return index;
}
  
  
/******************************************************************************/
/*!
 *  @function		getStatus
 *  @parameters		None
 *	@returns		EEPROM status register
 *	@description	Reads the status register from the EEPROM and returns the
 *					data
*/
unsigned char M95M02_getStatus (void) 
{
	unsigned char retVal = 0;
	EEPROM_CS;

	SendAndRcvByte(M95M02_CMD_RDSR);
	retVal = SendAndRcvByte(0);
			MULTI_CS_HIGH;

	return retVal;
}


/*!
 *  @function		commandWREN
 *  @parameters		None
 *	@returns		None
 *	@description	Sends the WRITE_ENABLE command to the EEPROM. This should
 *					happen immediately before sending the write command, 
 *					including when writing to the EEPROM's status register.
*/
void M95M02_commandWREN (void)
{
	EEPROM_CS;
    SendAndRcvByte(M95M02_CMD_WREN);
			MULTI_CS_HIGH;
}


/*!
 *  @function		commandWRITE
 *  @parameters		None
 *	@returns		None
 *	@description	Sends the WRITE command to the EEPROM. This should happen 
 *					immediately before sending data to be written, including 
 *					when writing to the EEPROM's status register.
*/
void commandWRITE (void)
{
    SendAndRcvByte(M95M02_CMD_WRITE);
	
}


/*!
 *  @function		commandREAD
 *  @parameters		None
 *	@returns		None
 *	@description	Sends the READ command to the EEPROM. This should happen 
 *					immediately before reading data from the EEPROM.
*/
void commandREAD (void)
{
    SendAndRcvByte(M95M02_CMD_READ);
	
}


/*!
 *  @function		getWIP
 *  @parameters		None
 *	@returns		WRITE_IN_PROGRESS flag from the EEPROM status register
 *	@description	Reads the EEPROM status register and returns the masked
 *					WIP flag. This function can be repeatedly called in a while
 *					loop to repeatedly poll the WIP flag until cleared for 
 *					faster write operation (as opposed to waiting in a delay
 *					loop for the worst-case write time).
*/
unsigned char M95M02_getWIP (void)
{
	return (M95M02_getStatus() & M95M02_STATUS_REGISTER_WIP_BIT);
}


/*!
 *  @function		readIDPage
 *  @parameters		address, pBuffer, size
 *	@returns		None
 *	@description	Reads the identification page to a buffer.
*/
void M95M02_readIDPage (unsigned char address, unsigned char * pBuffer, unsigned short size)
{unsigned short index;
	EEPROM_CS;
    SendAndRcvByte(M95M02_CMD_READ_ID);
    SendAndRcvByte(0x00);			// MSB
    SendAndRcvByte(0x00);			// Address bit A10 must be 0; upper address 
								// bits are Don't Care
    SendAndRcvByte(address);		// LSB The data byte pointed to by the lower 
								// address bits
	
    
    for(index = 0; index < size; index++)
	{
		pBuffer[index] = SendAndRcvByte(0);
	}
	MULTI_CS_HIGH;
}
unsigned char eeprom_id(void)
{
	unsigned char id = 0, pbuf[4];
	M95M02_readIDPage(90, pbuf, 4);
	id |= pbuf[3] | (pbuf[2] << 8) | (pbuf[1] << 16) | (pbuf[0] << 24);
	return id;
}

/*!
 *  @function		writeIDPage
 *  @parameters		address, pBuffer, size
 *	@returns		None
 *	@description	Writes the identification page from a buffer.
*/
void M95M02_writeIDPage (unsigned char address, unsigned char * pBuffer, unsigned short size)
{
    unsigned short index;
	
	EEPROM_CS;
    SendAndRcvByte(M95M02_CMD_WRITE_ID);
    SendAndRcvByte(0x00);			// MSB
    SendAndRcvByte(0x00);			// Address bit A10 must be 0; upper address 
								// bits are Don't Care
    SendAndRcvByte(address);		// LSB The data byte pointed to by the lower 
								// address bits
	
    for(index = 0; index < size; index++)
	{
    	SendAndRcvByte(pBuffer[index]);
	}
	MULTI_CS_HIGH;
}


/*!
 *  @function		readLockStatus
 *  @parameters		data
 *	@returns		None
 *	@description	Reads the lock status from the EEPROM into a buffer
*/
void M95M02_readLockStatus (unsigned char * data)
{EEPROM_CS;
	SendAndRcvByte(M95M02_CMD_READ_LOCK_STATUS);
	SendAndRcvByte(0x00);			// MSB
	SendAndRcvByte(0x04);			// Address bit A10 must be 1; all other address 
								// bits are Don't Care
	SendAndRcvByte(0x00);			// LSB
	*data = SendAndRcvByte(0);
	MULTI_CS_HIGH;
}


/*!
 *  @function		lockID
 *  @parameters		None
 *	@returns		None
 *	@description	Sends the lock ID command
*/
void M95M02_lockID (void)
{EEPROM_CS;
	SendAndRcvByte(M95M02_CMD_LOCK_ID);
	SendAndRcvByte(0x00);			// MSB
	SendAndRcvByte(0x04);			// Address bit A10 must be 1; all other address 
								// bits are Don't Care
	SendAndRcvByte(0x00);			// LSB
	SendAndRcvByte(0x02);			// The data byte sent must be equal to the 
								// binary value xxxx xx1x, where x = Don't Care
		MULTI_CS_HIGH;
}
