/*************************************************************************
 *
 *    Used with ICCARM and AARM.
 *
 *    (c) Copyright IAR Systems 2008
 *
 *    File name   : sd_spi_mode.c
 *    Description : SD/MMC driver
 *
 *    History :
 *    1. Date        : April 10, 2008
 *       Author      : Stanimir Bonev
 *       Description : Create
 *    $Revision: 31601 $
 **************************************************************************/

#define SD_SPI_MODE_GLOBAL

#include "sd_spi_mode.h"
#include "multiplexer.h"
#include "spi.h"
#include "itmr.h"
#include "uart.h"
#include <assert.h>

#define true	1
#define false	0
	




/* Following commands are SD Card Specific commands.
   SD_CMD_APP_CMD should be sent before sending these
   commands. */
#define SD_CMD_APP_SD_SET_BUSWIDTH                 ((unsigned char)6)  /* For SD Card only */
#define SD_CMD_SD_APP_STAUS                        ((unsigned char)13) /* For SD Card only */
#define SD_CMD_SD_APP_SEND_NUM_WRITE_BLOCKS        ((unsigned char)22) /* For SD Card only */
#define SD_CMD_SD_APP_OP_COND                      ((unsigned char)41) /* For SD Card only */
#define SD_CMD_SD_APP_SET_CLR_CARD_DETECT          ((unsigned char)42) /* For SD Card only */
#define SD_CMD_SD_APP_SEND_SCR                     ((unsigned char)51) /* For SD Card only */
#define SD_CMD_SD_CMD_RW_DIRECT                    ((unsigned char)52) /* For SD I/O Card only */
#define SD_CMD_SD_CMD_RW_EXTENDED                  ((unsigned char)53) /* For SD I/O Card only */

/* Following commands are SD Card Specific security commands.
   SD_CMD_APP_CMD should be sent before sending these commands. */
#define SD_CMD_SD_APP_GET_MKB                      ((unsigned char)43) /* For SD Card only */
#define SD_CMD_SD_APP_GET_MID                      ((unsigned char)44) /* For SD Card only */
#define SD_CMD_SD_APP_SET_CER_RN1                  ((unsigned char)45) /* For SD Card only */
#define SD_CMD_SD_APP_GET_CER_RN2                  ((unsigned char)46) /* For SD Card only */
#define SD_CMD_SD_APP_SET_CER_RES2                 ((unsigned char)47) /* For SD Card only */
#define SD_CMD_SD_APP_GET_CER_RES1                 ((unsigned char)48) /* For SD Card only */
#define SD_CMD_SD_APP_SECURE_READ_MULTIPLE_BLOCK   ((unsigned char)18) /* For SD Card only */
#define SD_CMD_SD_APP_SECURE_WRITE_MULTIPLE_BLOCK  ((unsigned char)25) /* For SD Card only */
#define SD_CMD_SD_APP_SECURE_ERASE                 ((unsigned char)38) /* For SD Card only */
#define SD_CMD_SD_APP_CHANGE_SECURE_AREA           ((unsigned char)49) /* For SD Card only */
#define SD_CMD_SD_APP_SECURE_WRITE_MKB             ((unsigned char)48) /* For SD Card only */

/* Masks for R6 Response */
#define SD_CHECK_PATTERN                ((unsigned int)0x000001AA)



/* SDIO Commands  Index */
#define SD_CMD_GO_IDLE_STATE                       ((unsigned char)0)
#define SD_CMD_SEND_OP_COND                        ((unsigned char)1)
#define SD_CMD_ALL_SEND_CID                        ((unsigned char)2)
#define SD_CMD_SET_REL_ADDR                        ((unsigned char)3) /* SD_CMD_SEND_REL_ADDR for SD Card */
#define SD_CMD_SET_DSR                             ((unsigned char)4)
#define SD_CMD_SD_CMD_SEN_OP_COND                  ((unsigned char)5)
#define SD_CMD_HS_SWITCH                           ((unsigned char)6)
#define SD_CMD_SEL_DESEL_CARD                      ((unsigned char)7)
#define SD_CMD_HS_SEND_EXT_CSD                     ((unsigned char)8)
#define SD_CMD_SEND_CSD                            ((unsigned char)9)
#define SD_CMD_SEND_CID                            ((unsigned char)10)
#define SD_CMD_READ_DAT_UNTIL_STOP                 ((unsigned char)11) /* SD Card doesn't support it */
#define SD_CMD_STOP_TRANSMISSION                   ((unsigned char)12)
#define SD_CMD_SEND_STATUS                         ((unsigned char)13)
#define SD_CMD_HS_BUSTEST_READ                     ((unsigned char)14)
#define SD_CMD_GO_INACTIVE_STATE                   ((unsigned char)15)
#define SD_CMD_SET_BLOCKLEN                        ((unsigned char)16)
#define SD_CMD_READ_SINGLE_BLOCK                   ((unsigned char)17)
#define SD_CMD_READ_MULT_BLOCK                     ((unsigned char)18)
#define SD_CMD_HS_BUSTEST_WRITE                    ((unsigned char)19)
#define SD_CMD_WRITE_DAT_UNTIL_STOP                ((unsigned char)20) /* SD Card doesn't support it */
#define SD_CMD_SET_BLOCK_COUNT                     ((unsigned char)23) /* SD Card doesn't support it */
#define SD_CMD_WRITE_SINGLE_BLOCK                  ((unsigned char)24)
#define SD_CMD_WRITE_MULT_BLOCK                    ((unsigned char)25)
#define SD_CMD_PROG_CID                            ((unsigned char)26) /* reserved for manufacturers */
#define SD_CMD_PROG_CSD                            ((unsigned char)27)
#define SD_CMD_SET_WRITE_PROT                      ((unsigned char)28)
#define SD_CMD_CLR_WRITE_PROT                      ((unsigned char)29)
#define SD_CMD_SEND_WRITE_PROT                     ((unsigned char)30)
#define SD_CMD_SD_ERASE_GRP_START                  ((unsigned char)32) /* To set the address of the first write
                                                                  block to be erased. (For SD card only) */
#define SD_CMD_SD_ERASE_GRP_END                    ((unsigned char)33) /* To set the address of the last write block of the
                                                                  continuous range to be erased. (For SD card only) */
#define SD_CMD_ERASE_GRP_START                     ((unsigned char)35) /* To set the address of the first write block to be erased.
                                                                  (For MMC card only spec 3.31) */

#define SD_CMD_ERASE_GRP_END                       ((unsigned char)36) /* To set the address of the last write block of the
                                                                  continuous range to be erased. (For MMC card only spec 3.31) */

#define SD_CMD_ERASE                               ((unsigned char)38)
#define SD_CMD_FAST_IO                             ((unsigned char)39) /* SD Card doesn't support it */
#define SD_CMD_GO_IRQ_STATE                        ((unsigned char)40) /* SD Card doesn't support it */
#define SD_CMD_LOCK_UNLOCK                         ((unsigned char)42)
#define SD_CMD_APP_CMD                             ((unsigned char)55)
#define SD_CMD_GEN_CMD                             ((unsigned char)56)
#define SD_CMD_READ_OCR                            ((unsigned char)58) /* Read OCR register */
#define SD_CMD_CRC_ON_OFF                          ((unsigned char)59) /* On/Off CRC check by SD Card */
#define SD_CMD_NO_CMD                              ((unsigned char)64)






// SD Maximum Block Rad Access Time
#define RD_TIME_OUT               100LL   // ms
// SD Maximum Block Write Access Time
#define WR_TIME_OUT               250LL   // ms

// Card R1 bitmap definitions
#define _SD_OK              0x00
#define _SD_ILDE_STATE      0x01
#define _SD_ERASE_RST       0x02
#define _SD_ILLEGAL_CMD     0x04
#define _SD_CRC_ERROR       0x08
#define _SD_ERASE_ERROR     0x10
#define _SD_ADD_ERROR       0x20
#define _SD_PARAM_ERROR     0x40

#define _SD_DATA_TOLKEN     0xFE
#define _SD_DATA_ERR_TOLKEN 0x1F
#define _SD_STOP_TRAN       0xFD

#define _CSD_GET_TRAN_SPEED_EXP()      (_SdSdCsd[ 3]&0x07)
#define _CSD_GET_TRAN_SPEED_MANT()    ((_SdSdCsd[ 3]&0xF8)>>3 )
#define _CSD_GET_NSAC()                (_SdSdCsd[ 2]          )
#define _CSD_GET_TAAC_EXP()            (_SdSdCsd[ 1]&0x7)
#define _CSD_GET_TAAC_MANT()          ((_SdSdCsd[ 1]&0xF8)>>3 )
#define _CSD_GET_R2W_FACTOR()         ((_SdSdCsd[12]&0x1C)>>2 )
#define _CSD_GET_READ_BL_LEN()         (_SdSdCsd[ 5]&0x0F)
#define _CSD_GET_C_SIZE()            (((_SdSdCsd[ 6]&0x03)<<10) + (_SdSdCsd[7]<<2) + ((_SdSdCsd[8]&0xc0)>>6))
#define _CSD_GET_C_SIZE_MULT()       (((_SdSdCsd[ 9]&0x03)<<1 ) +((_SdSdCsd[10]&0x80)>>7))
#define _CSD_GET_PERM_WRITE_PROTECT() ((_SdSdCsd[14]&0x20)>>5 )
#define _CSD_GET_TMP_WRITE_PROTECT()  ((_SdSdCsd[14]&0x10)>>4 )

#define _CSD_2_0_GET_C_SIZE()        (((_SdSdCsd[7]&0x0F)<<16) + (_SdSdCsd[8]<<8) + _SdSdCsd[9])

#define _OCR            0x003E0000
#define _HC             0x40000000
#define _CMD8_DATA      0x000001AA


#define SD_UNKNOWN_SD_CARD              ((unsigned int)0x0)
#define SD_STD_CAPACITY_SD_CARD_V1_0    ((unsigned int)0x1)
#define SD_STD_CAPACITY_SD_CARD_V2_0    ((unsigned int)0x2)
#define SD_MULTIMEDIA_CARD              ((unsigned int)0x3)
#define SD_HIGH_CAPACITY_SD_CARD        ((unsigned int)0x4)

unsigned char  SD_CardType = SD_UNKNOWN_SD_CARD;         // SD Card type
unsigned int SD_CardCapacity;                          // SD Card capacity
unsigned char  SD_MaxBusClkFreq = 0;                     // Max. card bus frequency
unsigned char  SD_MID = 0;                               // SD Card Maunufacturer ID
unsigned short SD_OID = 0;                               // SD Card OEM/Application ID


unsigned char  SD_CSD[16];                               // CSD buffer
unsigned char  SD_CID[16];                               // CID buffer
unsigned char  SD_sector[512];                           // sector buffer
unsigned short SD_CRC16_rcv;                             // last received CRC16
unsigned short SD_CRC16_cmp;                             // last computed CRC16


typedef enum __SdSpiCmdInd_t
{
  _CMD0 = 0,  // Resets the MultiMediaCard

  _CMD1,      // Activates the cards initialization process

  _CMD8,      // Send Interface Condition Command
              // for HC cards only

  _CMD9,      // Asks the selected card to send its card-specific data (CSD)

  _CMD10,     // Asks the selected card to send its card identification (CID)

  _CMD12,     // Stop transmission on multiple block read

  _CMD13,     // Asks the selected card to send its status register

  _CMD16,     // Selects a block length (in bytes) for all following block commands (read and write)

  _CMD17,     // Reads a block of the size selected by the SET_BLOCKLEN command

  _CMD18,     // Continuously transfers data blocks from card to host
              // until interrupted by a Stop command or the requested number of data blocks transmitted

  _CMD24,     // Writes a block of the size selected by the SET_BLOCKLEN command

  _CMD25,     // Continuously writes blocks of data until a Stop Tran
              // Token or the requested number of blocks received

  _CMD27,     // Programming of the programmable bits of the CSD

  _CMD28,     // If the card has write protection features, this
              // command sets the write protection bit of the
              // addressed group. The properties of write protection
              // are coded in the card specific data (WP_GRP_SIZE).

  _CMD29,     // If the card has write protection features, this
              // command clears the write protection bit of the addressed group

  _CMD30,     // If the card has write protection features, this
              // command asks the card to send the status of the write protection bits

  _CMD32,     // Sets the address of the first sector of the erase group

  _CMD33,     // Sets the address of the last sector in a continuous
              // range within the selected erase group, or the address
              // of a single sector to be selected for erase

  _CMD34,     // Removes one previously selected sector from the erase selection

  _CMD35,     // Sets the address of the first erase group within a range to be selected for erase

  _CMD36,     // Sets the address of the last erase group within a
              // continuous range to be selected for erase

  _CMD37,     // Removes one previously selected erase group from the erase selection.

  _CMD38,     // Erases all previously selected sectors

  _CMD42,     // Used to set/reset the password or lock/unlock the
              // card. The size of the Data Block is defined by the SET_BLOCK_LEN command

  _CMD55,     // Notifies the card that the next command is an
              // application specific command rather than a standard command.

  _CMD56,     // Used either to transfer a Data Block to the card or
              // to get a Data Block from the card for general
              // purpose/application specific commands. The size
              // of the Data Block is defined with SET_BLOCK_LEN command

  _CMD58,     // Reads the OCR register of a card

  _CMD59,     // Turns the CRC option on or off. A 1 in the CRC
              // option bit will turn the option on, a 0 will turn it off

  _ACMD41,    // Activates the cards initialization process (Only for SD)

  _CMD_END    // End of commands index
} _SdSpiCmdInd_t;



typedef double                Flo64;    // Double precision floating point
typedef double              * pFlo64;
typedef float                 Flo32;    // Single precision floating point
typedef float               * pFlo32;
typedef signed   long long    Int64S;   // Signed   64 bit quantity
typedef signed   long long  * pInt64S;
typedef unsigned long long    Int64U;   // Unsigned 64 bit quantity
typedef unsigned long long  * pInt64U;
typedef signed   int          Int32S;   // Signed   32 bit quantity
typedef signed   int        * pInt32S;
typedef unsigned int          Int32U;   // Unsigned 32 bit quantity
typedef unsigned int        * pInt32U;
typedef signed   short        Int16S;   // Signed   16 bit quantity
typedef signed   short      * pInt16S;
typedef unsigned short        Int16U;   // Unsigned 16 bit quantity
typedef unsigned short      * pInt16U;
typedef signed   char         Int8S;    // Signed    8 bit quantity
typedef signed   char       * pInt8S;
typedef unsigned char         Int8U;    // Unsigned  8 bit quantity
typedef unsigned char       * pInt8U;
typedef unsigned int          Boolean;  // Boolean
typedef unsigned int        * pBoolean;


typedef enum __SdAgmType_t
{
  _SdNoArg = 0, _SdBlockLen, _SdDataAdd, _SdDummyWord
} _SdAgmType_t;

typedef enum __SdRespType_t
{
  _SdR1 = 0, _SdR1b, _SdR2, _SdR3, _SdR7
} _SdRespType_t;

typedef struct __SdCommads_t
{
  Int8U         TxData;
  _SdAgmType_t  Arg;
  _SdRespType_t Resp;
} _SdCommads_t;

typedef enum __SdState_t
{
  _SdOk = 0, _SdNoPresent, _SdNoResponse, _SdCardError,
  _SdMiscompare, _SdUnsupported
} _SdState_t;


const Int32U _SdTransfExp[] =
{
     10000UL,
    100000UL,
   1000000UL,
  10000000UL,
         0UL,
         0UL,
         0UL,
         0UL,
};

const Int32U _SdAccessTime [] =
{
        1UL,
       10UL,
      100UL,
     1000UL,
    10000UL,
   100000UL,
  1000000UL,
 10000000UL,
};

const Int32U _SdCsdMant[] =
{
  0UL,10UL,12UL,13UL,15UL,
  20UL,25UL,
  30UL,35UL,
  40UL,45UL,
  50UL,55UL,
  60UL,
  70UL,
  80UL,
};

const _SdCommads_t _SdCmd[_CMD_END] =
{
  // CMD0
  {0x40,_SdNoArg    ,_SdR1 },
  // CMD1
  {0x41,_SdNoArg    ,_SdR1 },
  // CMD8
  {0x48,_SdDataAdd  ,_SdR7 },
  // CMD9
  {0x49,_SdNoArg    ,_SdR1 },
  // CMD10
  {0x4A,_SdNoArg    ,_SdR1 },
  // CMD12
  {0x4C,_SdNoArg    ,_SdR1 },
  // CMD13
  {0x4D,_SdNoArg    ,_SdR2 },
  // CMD16
  {0x50,_SdBlockLen ,_SdR1 },
  // CMD17
  {0x51,_SdDataAdd  ,_SdR1 },
  // CMD18
  {0x52,_SdDataAdd  ,_SdR1 },
  // CMD24
  {0x58,_SdDataAdd  ,_SdR1 },
  // CMD25
  {0x59,_SdDataAdd  ,_SdR1 },
  // CMD27
  {0x5B,_SdNoArg    ,_SdR1 },
  // CMD28
  {0x5C,_SdDataAdd  ,_SdR1b},
  // CMD29
  {0x5D,_SdDataAdd  ,_SdR1b},
  // CMD30
  {0x5E,_SdDataAdd  ,_SdR1 },
  // CMD32
  {0x60,_SdDataAdd  ,_SdR1 },
  // CMD33
  {0x61,_SdDataAdd  ,_SdR1 },
  // CMD34
  {0x62,_SdDataAdd  ,_SdR1 },
  // CMD35
  {0x63,_SdDataAdd  ,_SdR1 },
  // CMD36
  {0x64,_SdDataAdd  ,_SdR1 },
  // CMD37
  {0x65,_SdDataAdd  ,_SdR1 },
  // CMD38
  {0x66,_SdDummyWord,_SdR1b},
  // CMD42
  {0x6A,_SdDummyWord,_SdR1b},
  // CMD55
  {0x77,_SdNoArg    ,_SdR1 },
  // CMD56
  {0x78,_SdNoArg    ,_SdR1 },
  // CMD58
  {0x7A,_SdNoArg    ,_SdR3 },
  // CMD59
  {0x7B,_SdDummyWord,_SdR1 },
  // ACMD41
  {0x69,_SdDataAdd  ,_SdR1 }
};

#define _CS_L() MICROSD_CS
#define _CS_H() MULTI_CS_HIGH; SendAndRcvByte(0xFF)		//!!!

static Int32U _SdSendCmd(_SdSpiCmdInd_t ComdInd, pInt32U pData);
static void _SdCsdImplemet (void);
static _SdState_t _SdInitMedia (void);

static   _SdState_t _SdReadCardInfo(pInt8U pData);
static   _SdState_t _SdRead(pInt8U pData, Int32U Add);
static   _SdState_t _SdWrite(pInt8U pData, Int32U Add);
static   _SdState_t _SdVerify(pInt8U pData, Int32U Add);

static DiskCtrlBlk_t _SdDskCtrlBlk;
static Int32U        Tnac;
static Int32U        _Twr;
static Int8U         _SdSdCsd[16];
static Boolean       _bSdPermWriteProtect;
static Boolean       _bHC;



/*************************************************************************
 * Function Name: SdWriteProtect
 * Parameters: none
 * Return: Boolean - true cart is protected
 *                 - false cart no protected
 *
 * Description: SD/MMC Write protect check
 *
 *************************************************************************/
Boolean SdWriteProtect (void)
{
  return false;
}

/*************************************************************************
 * Function Name: SdSendBlock
 * Parameters: pInt8U pData, Int32U Size
 *
 * Return: void
 *
 * Description: Read byte from SPI
 *
 *************************************************************************/
void SdSendBlock (pInt8U pData, Int32U Size)
{
	unsigned int Dummy;
  unsigned int OutCount = Size;
  while (OutCount--)
  {
		SendAndRcvByte(*pData++);
    Dummy = SendAndRcvByte(0);
  }
}

/*************************************************************************
 * Function Name: SdReceiveBlock
 * Parameters: pInt8U pData, Int32U Size
 *
 * Return: void
 *
 * Description: Read byte from SPI
 *
 *************************************************************************/
void SdReceiveBlock (pInt8U pData, Int32U Size)
{
unsigned int InCount = Size;
  while (InCount--)
  {
		SendAndRcvByte(0xFF);
    pData += SendAndRcvByte(0);
  }
}


/*************************************************************************
 * Function Name: _SdSendCmd
 * Parameters: _SdSpiCmdInd_t ComdInd, Int32U Arg
 *
 * Return: Int32U
 *
 * Description: SD/MMC commands implement
 *
 *************************************************************************/
static Int32U _SdSendCmd(_SdSpiCmdInd_t ComdInd, pInt32U pData)
{
Int32U ch = 0xFF;
Int32U i;
  // Chip Select
  _CS_L();
  // Send command code
  SendAndRcvByte(_SdCmd[ComdInd].TxData);
  // Send command's arguments
  if(_SdCmd[ComdInd].Arg == _SdNoArg)
  {
    SendAndRcvByte(0x00);
    SendAndRcvByte(0x00);
    SendAndRcvByte(0x00);
    SendAndRcvByte(0x00);
  }
  else
  {
    //assert(pData);
    SendAndRcvByte(*pData >> 24);
    SendAndRcvByte(*pData >> 16);
    SendAndRcvByte(*pData >> 8 );
    SendAndRcvByte(*pData      );
  }
  // Send CRC
  if(_CMD0 == ComdInd)
  {
    SendAndRcvByte(0x95);
  }
  else
  {
    SendAndRcvByte(0xFF);
  }
  // wait for command response
  for(i = 9; i && (ch == 0xFF); --i) ch = SendAndRcvByte(0xFF);

  if (0 == i)
  {
    _CS_H();
    return((Int32U)-1);
  }

  // Implement command response
  switch (_SdCmd[ComdInd].Resp)
  {
  case _SdR1b:
    for (ch = 0,i = _Twr; i && (ch != 0xFF); --i)
    {
      ch = SendAndRcvByte(0xFF);
    }
  case _SdR1:
    break;
  case _SdR2:
    ch  = ((Int32U)ch << 8)     & 0x0000FF00;
    ch |= SendAndRcvByte(0xFF)  & 0x000000FF;
    break;
  default:
    //assert(pData);
    *pData  = ((Int32U)SendAndRcvByte(0xFF) << 24) & 0xFF000000;
    *pData |= ((Int32U)SendAndRcvByte(0xFF) << 16) & 0x00FF0000;
    *pData |= ((Int32U)SendAndRcvByte(0xFF) << 8 ) & 0x0000FF00;
    *pData |=          SendAndRcvByte(0xFF)        & 0x000000FF;
  }
  return(ch);
}

/*************************************************************************
 * Function Name: _SdInitMedia
 * Parameters: none
 *
 * Return: _SdState_t
 *
 * Description: SD/MMC detect and initialize
 *
 *************************************************************************/
static _SdState_t _SdInitMedia (void)
{
Int32U i,res, data;
  Tnac = 1;
  /*for (i=100; i; i--); // small delay
  if(!SdPresent())
  {
    SdPowerOff();
    return(_SdNoPresent);
  }
  // Clock Freq. Identification Mode < 400kHz
  SdSetClockFreq(IdentificationModeClock);
  //Enable MMC/SD power
  SdPowerOn();
  // After power up at least 74 clock cycles are required prior to
  // starting bus communication
  _CS_H();*/
  itmr_Delay(1);
   _CS_L();
  for(i = 10; i; --i) SendAndRcvByte(0xFF);
	_CS_H();
  // CMD0 (Go to IDLE) and put MMC/SD card in SPI mode
  res = _SdSendCmd(_CMD0,0);
  _CS_H();
  if(_SD_ILDE_STATE != res)
  {
    return(_SdNoResponse);
  }

  // Determinate Card type - SD Class 1, SD Class2 or MMC
  _SdDskCtrlBlk.DiskType = DiskSD_Spec2_0;
  _bHC = false;

  for(i=100;i;--i)
  {
    data = _CMD8_DATA;
    res = _SdSendCmd(_CMD8,&data);
    _CS_H();
    if(res & _SD_ILLEGAL_CMD)
    {
      // The card doesn't support specification 2.0
       _SdDskCtrlBlk.DiskType = DiskSD_Spec1_x;
    }
    else if (  (_CMD8_DATA & 0xFF) == (data & 0xFF)
             && 0 == ((1<<8) & data))
    {
      return(_SdUnsupported);
    }

    res = _SdSendCmd(_CMD58,&data);
    _CS_H();
    
    // SPP +
    // datasheet: Production Manual SDCard v1.9, chapter 3.11 page 37:
    // "If bit 32 (the busy bit) is set, it informs the host 
    // that the card power up procedure is finished."
    // In this case we don't have to check the returned res.
    if (!(data & 0x80000000))
    // SPP -
      if (~_SD_ILDE_STATE & res)
      {
        // get CID must be always valid
        return(_SdNoResponse);
      }

    if (0 == (_OCR & data))
    {
      return(_SdUnsupported);
    }

    res = _SdSendCmd(_CMD55,0);
    _CS_H();
    data = (DiskSD_Spec2_0 == _SdDskCtrlBlk.DiskType)?_HC:0;
    res = _SdSendCmd(_ACMD41,&data);
    _CS_H();
    if(_SD_ILLEGAL_CMD & res)
    {
      // MMC card may be
      _SdDskCtrlBlk.DiskType = DiskMMC;
      break;
    }
    if(_SD_OK == res)
    {
      // SD card is find
      // Get Card's capacity type
      res = _SdSendCmd(_CMD58,&data);
      _CS_H();
      if(~_SD_ILDE_STATE & res)
      {
        // get CID must be always valid
        return(_SdNoResponse);
      }

      _bHC = 0 != (_HC & data);
      break;
    }
    itmr_Delay(50);
  }
  if(i == 0)
  {
    return(_SdNoResponse);
  }

  if(DiskMMC == _SdDskCtrlBlk.DiskType)
  {
    // CMD1 for MMC Init sequence
    // will be complete within 500ms
    for (i = 100; i;--i)
    {
      res = _SdSendCmd(_CMD1,0);
      _CS_H();
      if(_SD_OK == res)
      {
        // Init complete
        break;
      }
      itmr_Delay(50);
    }
    if(i == 0)
    {
      return(_SdNoResponse);
    }
  }

  // Read CSD
  res = _SdReadCardInfo(_SdSdCsd);
  if(_SD_OK != res)
  {
    // CSD must be always valid
    return(_SdNoResponse);
  }

  _SdCsdImplemet();

  res = _SdSendCmd(_CMD16,&_SdDskCtrlBlk.BlockSize);
  if(_SD_OK != res)
  {
    // CSD must be always valid
    return(_SdNoResponse);
  }
  _CS_H();
  return(_SdOk);
}

/*************************************************************************
 * Function Name: _SdReadCardInfo
 * Parameters: pInt8U pData,
 *
 * Return: _SdState_t
 *
 * Description: Read CSD registers depend of command
 *
 *************************************************************************/
static   _SdState_t _SdReadCardInfo(pInt8U pData)
{
Int32U i;
Int32U res;
  res = _SdSendCmd(_CMD9,0);
  if (res == _SD_OK)
  {
    for(i = 8; i ; --i)
    {
      res = SendAndRcvByte(0xFF);
      if(_SD_DATA_ERR_TOLKEN == (res | _SD_DATA_ERR_TOLKEN))
      {
        return(_SdCardError);
      }
      else if (_SD_DATA_TOLKEN == res)
      {
        // Read CSD or CID data
        for(i = 0; 16 > i; ++i)
        {
          *pData++ = SendAndRcvByte(0xFF);
        }
        // CRC receive
        SendAndRcvByte(0xFF);
        SendAndRcvByte(0xFF);
        _CS_H();
        return(_SdOk);
      }
    }
  }
  _CS_H();
  return(_SdNoResponse);
}

/*************************************************************************
 * Function Name: _SdCsdImplemet
 * Parameters:  none
 *
 * Return: none
 *
 * Description: Implement data from CSD
 *
 *************************************************************************/
static void _SdCsdImplemet (void)
{
Int32U Freq;
unsigned long Tmp;
  // Calculate SPI max clock
 // Freq = _SdTransfExp[_CSD_GET_TRAN_SPEED_EXP()] * _SdCsdMant[_CSD_GET_TRAN_SPEED_MANT()];
 // Freq = SdSetClockFreq(Freq);
 Freq = 4000000;
  if(DiskMMC == _SdDskCtrlBlk.DiskType)
  {
    // Calculate Time outs for MMC cards
    Tmp = _SdAccessTime[_CSD_GET_TAAC_EXP()] * _SdCsdMant[_CSD_GET_TAAC_MANT()];
    Tmp /= 10000; // us
    // Freq [Hz], Tmp[1 us], *10
    Tmp = (Freq*Tmp)/100000LL;
    // NSAC*100*10
    Tmp += 1000*_CSD_GET_NSAC();
    // Max time out
    Tnac = Tmp;
    _Twr  = Tmp * (1<<_CSD_GET_R2W_FACTOR());
  }
  else
  {
    // Calculate Time outs for SD cards
    // Freq [Hz], RD_TIME_OUT[ms]
    Tmp = Freq/1000;
    Tnac = Tmp * RD_TIME_OUT;
    _Twr  = Tmp * WR_TIME_OUT;
  }
  // Calculate Block size and Block Number
  _SdDskCtrlBlk.BlockSize = 1<<_CSD_GET_READ_BL_LEN();
  if(_bHC)
  {
    _SdDskCtrlBlk.BlockNumb = (_CSD_2_0_GET_C_SIZE()+1) * 1024;
  }
  else
  {
    _SdDskCtrlBlk.BlockNumb = (_CSD_GET_C_SIZE()+1)*(4<<_CSD_GET_C_SIZE_MULT());
  }
  if(_SdDskCtrlBlk.BlockSize != SD_DEF_BLK_SIZE)
  {
    // because Windows support only 512 bytes block
    _SdDskCtrlBlk.BlockNumb *= (_SdDskCtrlBlk.BlockSize>>9);
    _SdDskCtrlBlk.BlockSize  = SD_DEF_BLK_SIZE;
  }
  // Set Write Protect
  _bSdPermWriteProtect = _CSD_GET_PERM_WRITE_PROTECT() |\
                          _CSD_GET_TMP_WRITE_PROTECT();
  _SdDskCtrlBlk.WriteProtect = SdWriteProtect() |\
                              _bSdPermWriteProtect;
}

/*************************************************************************
 * Function Name: _SdRead
 * Parameters: pInt8U pData, Int32U Add, Int32U Length
 *
 * Return: _SdState_t
 *
 * Description: Read from a SD/MMC
 *
 *************************************************************************/
static   _SdState_t _SdRead(pInt8U pData, Int32U Add)
{
Int32U res,i;
  res = _SdSendCmd(_CMD17,&Add);
  if(res == _SD_OK)
  {
    for(i = Tnac; i; --i)
    {
      res = SendAndRcvByte(0xFF);
      if(_SD_DATA_ERR_TOLKEN == (res | _SD_DATA_ERR_TOLKEN))
      {
        _CS_H();
        return(_SdCardError);
      }
      else if (res == _SD_DATA_TOLKEN)
      {
        // Receive block
        SdReceiveBlock(pData, SD_DEF_BLK_SIZE);
        // CRC receive
        SendAndRcvByte(0xFF);
        SendAndRcvByte(0xFF);
        _CS_H();
        return(_SdOk);
      }
    }
  }
  _CS_H();
  return(_SdNoResponse);
}

/*************************************************************************
 * Function Name: _SdWrite
 * Parameters: pInt8U pData, Int32U Add, Int32U Length
 *
 * Return: _SdState_t
 *
 * Description: Write to a SD/MMC
 *
 *************************************************************************/
static   _SdState_t _SdWrite(pInt8U pData, Int32U Add)
{
	int Timeout;
	Int32U res,i;
  res = _SdSendCmd(_CMD24,&Add);
  if(_SD_OK == res)
  {
    res = SendAndRcvByte(0xFF);
    res = SendAndRcvByte(_SD_DATA_TOLKEN);
    // Send block
    SdSendBlock(pData,SD_DEF_BLK_SIZE);
    // CRC Send
    res = SendAndRcvByte(0xFF);
#if 0 // original not working
    SendAndRcvByte(0xFF);
    res = SendAndRcvByte(0xFF) & 0x1F;
    if(res != 0x05)
#else // SPP - work
    Timeout = 10;
    do
    {
      res = SendAndRcvByte(0xFF) & 0x1F;
    }
    while (res != 0x05 && Timeout--);
    if (!Timeout)
#endif
    {
      _CS_H();
      return(_SdCardError);
    }
    for(i = _Twr; i ;i--)
    {
      if(SendAndRcvByte(0xFF) == 0xFF)
      {
        break;
      }
    }

    _CS_H();
    if (0 == i)
    {
      return(_SdNoResponse);
    }
    return(_SdOk);
  }
  _CS_H();
  return(_SdNoResponse);
}

/*************************************************************************
 * Function Name: _SdVerify
 * Parameters: pInt8U pData, Int32U Add, Int32U Length
 *
 * Return: _SdState_t
 *
 * Description: Verify on a SD/MMC
 *
 *************************************************************************/
 static   _SdState_t _SdVerify(pInt8U pData, Int32U Add)
{
Int32U res,i;
  res = _SdSendCmd(_CMD17,&Add);
  if(res == _SD_OK)
  {
    for(i = Tnac;i;--i)
    {
      res = SendAndRcvByte(0xFF);
      if(_SD_DATA_ERR_TOLKEN == (res | _SD_DATA_ERR_TOLKEN))
      {
        return(_SdCardError);
      }
      else if (_SD_DATA_TOLKEN == res)
      {
        res = 0;
        for(i = 0; i<SD_DEF_BLK_SIZE;++i,++pData)
        {
          *pData ^= SendAndRcvByte(0xFF);
          if (*pData != 0)
          {
            res = 1;
          }
        }
        // CRC receive
        SendAndRcvByte(0xFF);
        SendAndRcvByte(0xFF);
        _CS_H();
        if (res)
        {
          return(_SdMiscompare);
        }
        return(_SdOk);
      }
    }
  }
  return(_SdNoResponse);
}

/*************************************************************************
 * Function Name: SdStatusUpdate
 * Parameters: none
 *
 * Return: none
 *
 * Description: Update status of SD/MMC card
 *
 *************************************************************************/
void SdStatusUpdate (void)
{
   int i;
  // Update WP state
  _SdDskCtrlBlk.WriteProtect = SdWriteProtect() |\
                               _bSdPermWriteProtect;
  
  for (i=100; i; i--); // small delay
 /* if(!SdPresent())
  {
    _SdDskCtrlBlk.DiskStatus = DiskNotPresent;
    SdPowerOff();
    return;
  }*/
  if((DiskCommandPass != _SdDskCtrlBlk.DiskStatus))
  {
    switch (_SdInitMedia())
    {
    case _SdOk:
      _SdDskCtrlBlk.DiskStatus = DiskCommandPass;
      _SdDskCtrlBlk.MediaChanged = true;
      break;
    case _SdCardError:
      _SdDskCtrlBlk.DiskStatus = DiskNotReady;
      break;
    case _SdNoResponse:
      _SdDskCtrlBlk.DiskStatus = DiskUknowError;
    break;
    default:
      _SdDskCtrlBlk.DiskStatus = DiskNotPresent;
      break;
    }
  }
  else if (_SdOk != _SdReadCardInfo(_SdSdCsd))
  {
    _SdDskCtrlBlk.DiskStatus = DiskNotReady;
  }
}

/*************************************************************************
 * Function Name: SdDiskInit
 * Parameters:  none
 *
 * Return: none
 *
 * Description: Init MMC/SD disk
 *
 *************************************************************************/
void SdDiskInit (void)
{
  _SdDskCtrlBlk.BlockNumb =\
  _SdDskCtrlBlk.BlockSize = 0;
  // Init SPI
  //SdInit();
  // Media Init
  switch (_SdInitMedia())
  {
  case _SdOk:
    _SdDskCtrlBlk.DiskStatus = DiskCommandPass;
    _SdDskCtrlBlk.MediaChanged = true;		
    break;
  case _SdCardError:
    _SdDskCtrlBlk.DiskStatus = DiskNotReady;
    break;
  case _SdNoResponse:
      _SdDskCtrlBlk.DiskStatus = DiskUknowError;
    break;
  default:
    _SdDskCtrlBlk.DiskStatus = DiskNotPresent;
    break;
  }
}

/*************************************************************************
 * Function Name: SdGetDiskCtrlBkl
 * Parameters:  none
 *
 * Return: pDiskCtrlBlk_t
 *
 * Description: Return pointer to status structure of the disk
 *
 *************************************************************************/
pDiskCtrlBlk_t SdGetDiskCtrlBkl (void)
{
  return(&_SdDskCtrlBlk);
}

/*************************************************************************
 * Function Name: SdDiskIO
 * Parameters: pInt8U pData,Int32U BlockStart,
 *             Int32U BlockNum, DiskIoRequest_t IoRequest
 *
 * Return: DiskStatusCode_t
 *
 * Description: MMC/SD disk I/O
 *
 *************************************************************************/
DiskStatusCode_t SdDiskIO (pInt8U pData,Int32U BlockStart, Int32U BlockNum, DiskIoRequest_t IoRequest)
{
  if((0 == pData) || (BlockStart+BlockNum > _SdDskCtrlBlk.BlockNumb))
  {
    return(DiskParametersError);
  }
  if (_SdDskCtrlBlk.DiskStatus)
  {
    return(_SdDskCtrlBlk.DiskStatus);
  }
  switch (IoRequest)
  {
  case DiskWrite:
    if(_SdDskCtrlBlk.WriteProtect)
    {
      return(DiskParametersError);
    }
    while(BlockNum--)
    {
      switch (_SdWrite(pData,_bHC?BlockStart:BlockStart*_SdDskCtrlBlk.BlockSize))
      {
      case _SdOk:
        BlockStart = _bHC?++BlockStart:BlockStart+_SdDskCtrlBlk.BlockSize;
        break;
      case _SdCardError:
        _SdDskCtrlBlk.DiskStatus = DiskNotReady;
        return(_SdDskCtrlBlk.DiskStatus);
      case _SdNoResponse:
        _SdDskCtrlBlk.DiskStatus = DiskUknowError;
        return(_SdDskCtrlBlk.DiskStatus);
      default:
        _SdDskCtrlBlk.DiskStatus = DiskNotPresent;
        return(_SdDskCtrlBlk.DiskStatus);
      }
    }
    break;
  case DiskRead:
    while(BlockNum--)
    {
      switch (_SdRead(pData,
                     _bHC?BlockStart:BlockStart*_SdDskCtrlBlk.BlockSize))
      {
      case _SdOk:
        BlockStart = _bHC?++BlockStart:BlockStart+_SdDskCtrlBlk.BlockSize;
        break;
      case _SdCardError:
        _SdDskCtrlBlk.DiskStatus = DiskNotReady;
        return(_SdDskCtrlBlk.DiskStatus);
      case _SdNoResponse:
        _SdDskCtrlBlk.DiskStatus = DiskUknowError;
        return(_SdDskCtrlBlk.DiskStatus);
      default:
        _SdDskCtrlBlk.DiskStatus = DiskNotPresent;
        return(_SdDskCtrlBlk.DiskStatus);
      }
    }
    break;
  case DiskVerify:
    while(BlockNum--)
    {
      switch (_SdVerify(pData,
                        _bHC?BlockStart:BlockStart*_SdDskCtrlBlk.BlockSize))
      {
      case _SdOk:
        BlockStart = _bHC?++BlockStart:BlockStart+_SdDskCtrlBlk.BlockSize;
        break;
      case _SdCardError:
        _SdDskCtrlBlk.DiskStatus = DiskNotReady;
        return(_SdDskCtrlBlk.DiskStatus);
      case _SdNoResponse:
        _SdDskCtrlBlk.DiskStatus = DiskUknowError;
        return(_SdDskCtrlBlk.DiskStatus);
      default:
        _SdDskCtrlBlk.DiskStatus = DiskNotPresent;
        return(_SdDskCtrlBlk.DiskStatus);
      }
    }
    break;
  default:
    return(DiskParametersError);
  }
  return(_SdDskCtrlBlk.DiskStatus);
}
// OTHER SD COMMANDS


static unsigned short CRC16_one(unsigned short crc, unsigned char ser_data) {
	crc  = (unsigned char)(crc >> 8)|(crc << 8);
	crc ^= ser_data;
	crc ^= (unsigned char)(crc & 0xff) >> 4;
	crc ^= (crc << 8) << 4;
	crc ^= ((crc & 0xff) << 4) << 1;

	return crc;
}
unsigned short CRC16_buf(const unsigned char * p, unsigned int len) {
	unsigned int i;
	unsigned short crc = 0;

	for (i = 0; i < len; i++) crc = CRC16_one(crc,p[i]);

	return crc;
}


static unsigned char CRC7_one(unsigned char t, unsigned char data) {
	const unsigned char g = 0x89;
	unsigned char i;

	t ^= data;
	for (i = 0; i < 8; i++) {
		if (t & 0x80) t ^= g;
		t <<= 1;
	}

	return t;
}
unsigned char SD_SendCmd(unsigned char cmd, unsigned int arg) {
	unsigned char wait, response, crc = 0;

	SendAndRcvByte(0xff); // This dummy send necessary for some cards

	// Send: [8b]Command -> [32b]Argument -> [8b]CRC
	SendAndRcvByte(cmd | 0x40);
	crc = CRC7_one(crc,cmd | 0x40);
	SendAndRcvByte(arg >> 24);
	crc = CRC7_one(crc,arg >> 24);
	SendAndRcvByte(arg >> 16);
	crc = CRC7_one(crc,arg >> 16);
	SendAndRcvByte(arg >> 8);
	crc = CRC7_one(crc,arg >> 8);
	SendAndRcvByte(arg);
	crc = CRC7_one(crc,arg);
	SendAndRcvByte(crc | 0x01); // Bit 1 always must be set to "1" in CRC

	// Wait for response from SD Card
	wait = 0;
	while ((response = SendAndRcvByte(0xff)) == 0xff) if (wait++ > 200) break;

	return response;
}

// return values:
//   0xff - SD card timeout
//   0xfe - Unknown or bad SD/MMC card
//   0xfd - Set block size command failed
//   0xfc - bad response for CMD58
//   0xfb - SDv2 pattern mismatch (in response to CMD8)
unsigned char SD_CardInit(void) {
	unsigned int wait = 0, r3;
	unsigned char response;

	MICROSD_CS;

	// Must send at least 74 clock ticks to SD Card
	for (wait = 0; wait < 8; wait++) SendAndRcvByte(0xff);

	// Software SD Card reset
	wait = 0; response = 0x00;
	while (wait < 0x20 && response != 0x01) {
		// Wait for SD card enters idle state (R1 response = 0x01)
		response = SD_SendCmd(SD_CMD_GO_IDLE_STATE,0x00);
		wait++;
	}
	if (wait >= 0x20 && response != 0x01) return 0xff; // SD card timeout

	// CMD8: SEND_IF_COND. Send this command to verify SD card interface operating condition
    /* Argument: - [31:12]: Reserved (shall be set to '0')
	             - [11:8]: Supply Voltage (VHS) 0x1 (Range: 2.7-3.6 V)
	             - [7:0]: Check Pattern (recommended 0xAA) */
	response = SD_SendCmd(SD_CMD_HS_SEND_EXT_CSD,SD_CHECK_PATTERN); // CMD8

	if (response == 0x01) {
		// SDv2 or later
		// Read R7 responce
		r3 =  SendAndRcvByte(0xff) << 24;
		r3 |= SendAndRcvByte(0xff) << 16;
		r3 |= SendAndRcvByte(0xff) << 8;
		r3 |= SendAndRcvByte(0xff);

		if ((r3 & 0x01ff) != (SD_CHECK_PATTERN & 0x01ff)) return 0xfb; // SDv2 pattern mismatch -> unsupported SD card

		// CMD55: Send leading command for ACMD<n> command.
		// CMD41: APP_SEND_OP_COND. For only SDC - initiate initialization process.
		wait = 0; response = 0xff;
		while (++wait < 0x2710 && response != 0x00) {
			SD_SendCmd(SD_CMD_APP_CMD,0); // CMD55
			response = SD_SendCmd(SD_CMD_SD_APP_OP_COND,0x40000000); // ACMD41: HCS flag set
		}
		if (wait >= 0x2710 || response != 0x00) return 0xff; // SD card timeout

		SD_CardType = SD_STD_CAPACITY_SD_CARD_V2_0; // SDv2;

		// Read OCR register
		response = SD_SendCmd(SD_CMD_READ_OCR,0x00000000); // CMD58
		if (response == 0x00) {
			// Get R3 response
			r3  = SendAndRcvByte(0xff) << 24;
			r3 |= SendAndRcvByte(0xff) << 16;
			r3 |= SendAndRcvByte(0xff) << 8;
			r3 |= SendAndRcvByte(0xff);
		} else {
			SD_CardType = SD_UNKNOWN_SD_CARD;
			return 0xfc; // bad CMD58 response
		}
		if (r3 & (1<<30)) SD_CardType = SD_HIGH_CAPACITY_SD_CARD; // SDHC or SDXC
	} else {
		// SDv1 or MMC
		wait = 0; response = 0xff;
		while (++wait < 0xfe) {
			SD_SendCmd(SD_CMD_APP_CMD,0); // CMD55
			response = SD_SendCmd(SD_CMD_SD_APP_OP_COND,0x00000000); // CMD41
			if (response == 0x00) {
				SD_CardType = SD_STD_CAPACITY_SD_CARD_V1_0; // SDv1
				break;
			}
		}

		if (response == 0x05 && wait >= 0xfe) {
			// MMC or bad card
			// CMD1: Initiate initialization process.
			wait = 0; response = 0xff;
			while (++wait < 0xfe) {
				response = SD_SendCmd(SD_CMD_SEND_OP_COND,0x00000000); // CMD1
				if (response == 0x00) {
					SD_CardType = SD_MULTIMEDIA_CARD; // MMC
					break;
				}
			}
		}
	}

	if (SD_CardType == 0) return 0xfe; // Unknown or bad SD/MMC card

	// Set SPI to higher speed
	//SD_SPI_Init(SPI_BaudRatePrescaler_8);

	// Turn off CRC
	response = SD_SendCmd(SD_CMD_CRC_ON_OFF,0x00000001); // CMD59

	// For SDv2,SDv1,MMC must set block size. For SDHC/SDXC it fixed to 512.
	if ((SD_CardType == SD_STD_CAPACITY_SD_CARD_V1_0) ||
		(SD_CardType == SD_STD_CAPACITY_SD_CARD_V2_0) ||
		(SD_CardType == SD_MULTIMEDIA_CARD))
	{
		response = SD_SendCmd(SD_CMD_SET_BLOCKLEN,0x00000200); // CMD16: block size = 512 bytes
		if (response != 0x00) return 0xfd; // Set block size failed
	}

	MULTI_CS_HIGH;

	return 0;
}

// return:
// 0x00 -- read OK
// 0x01..0xfe -- error response to CMD9
// 0xff -- timeout
unsigned char SD_Read_CSD(void) {
	unsigned int wait;
	unsigned char i, response;
unsigned int c_size,c_size_mult;
	MICROSD_CS;

	response = SD_SendCmd(SD_CMD_SEND_CSD,0); // CMD9
	if (response != 0x00) {
		// Something wrong happened, fill buffer with zeroes
		for (i = 0; i < 16; i++) SD_CSD[i] = 0x00;
		return response;
	} else {
		wait = 0; response = 0;
		while (++wait <= 0x1ff && response != 0xfe)	response = SendAndRcvByte(0xff);
		if (wait >= 0x1ff) return 0xff;
		// Read 16 bytes of CSD register
		for (i = 0; i < 16; i++) SD_CSD[i] = SendAndRcvByte(0xff);
	}

	// Receive 16-bit CRC (some cards demand this)
	SD_CRC16_rcv  = SendAndRcvByte(0xff) << 8;
	SD_CRC16_rcv |= SendAndRcvByte(0xff);

	// // Calculate CRC16 of received buffer
	SD_CRC16_cmp = CRC16_buf(&SD_CSD[0],16);

	MULTI_CS_HIGH;

	// Parse some stuff from CID
	SD_MaxBusClkFreq = SD_CSD[3];
	
	if (SD_CardType != SD_MULTIMEDIA_CARD) {
		if (SD_CSD[0] >> 6 == 0x01) {
			// CSD Version 2.0
			c_size  = (SD_CSD[7] & 0x3f) << 16;
			c_size |= SD_CSD[8] << 8;
			c_size |= SD_CSD[7];
			SD_CardCapacity = c_size << 9; // = c_size * 512
		} else {
			// CSD Version 1.0
			c_size  = (SD_CSD[6] & 0x03) << 10;
			c_size |= (unsigned int)SD_CSD[7] << 2;
			c_size |= SD_CSD[8] >> 6;
			c_size_mult  = (SD_CSD[9] & 0x03) << 1;
			c_size_mult |= SD_CSD[10] >> 7;
			SD_CardCapacity = c_size << 10;
		}
	} else {
		SD_CardCapacity = 0;
	}

	return 0;
}

// return:
// 0x00 -- read OK
// 0x01..0xfe -- error response to CMD10
// 0xff -- timeout
unsigned char SD_Read_CID(void) {
	unsigned int wait;
	unsigned char i, response;

	MICROSD_CS;

	response = SD_SendCmd(SD_CMD_SEND_CID,0); // CMD10
	if (response != 0x00) {
		// Something wrong happened, fill buffer with zeroes
		for (i = 0; i < 16; i++) SD_CID[i] = 0x00;
		return response;
	} else {
		wait = 0; response = 0;
		while (++wait <= 0x1ff && response != 0xfe)	response = SendAndRcvByte(0xff);
		if (wait >= 0x1ff) return 0xff;
		// Read 16 bytes of CID register
		for (i = 0; i < 16; i++) SD_CID[i] = SendAndRcvByte(0xff);
	}

	// Receive 16-bit CRC (some cards demand this)
	SD_CRC16_rcv  = SendAndRcvByte(0xff) << 8;
	SD_CRC16_rcv |= SendAndRcvByte(0xff);

	// // Calculate CRC16 of received buffer
	SD_CRC16_cmp = CRC16_buf(&SD_CID[0],16);

	MULTI_CS_HIGH;

	return 0;
}

// return:
// 0x00 -- read OK
// 0x01..0xfe -- error response from CMD17
// 0xff -- timeout
unsigned char SD_Read_Block(unsigned int addr) {
	unsigned int wait;
	unsigned short i;
	unsigned char response;

	MICROSD_CS;

	if (SD_CardType != SD_HIGH_CAPACITY_SD_CARD) addr <<= 9; // Convert block number to byte offset
	response = SD_SendCmd(SD_CMD_READ_SINGLE_BLOCK,addr); // CMD17
	if (response != 0x00) {
		// Something wrong happened, fill buffer with zeroes
		for (i = 0; i < 512; i++) SD_sector[i] = 0;
		return response; // SD_CMD_READ_SINGLE_BLOCK command returns bad response
	} else {
		wait = 0; response = 0;
		while (++wait <= 0x1ff && response != 0xfe)	response = SendAndRcvByte(0xff);
		if (wait >= 0x1ff) return 0xff;
		// Read 512 bytes of sector
		for (i = 0; i < 512; i++) SD_sector[i] = SendAndRcvByte(0xff);
	}

	// Receive 16-bit CRC (some cards demand this)
	SD_CRC16_rcv  = SendAndRcvByte(0xff) << 8;
	SD_CRC16_rcv |= SendAndRcvByte(0xff);

	// // Calculate CRC16 of received buffer
	SD_CRC16_cmp = CRC16_buf(&SD_sector[0],512);

	MULTI_CS_HIGH;

	return 0;
}
