#include "sdcard.h"
#include "spi.h"
#include "multiplexer.h"
#include "uart.h"


unsigned char  SD_CardType = SD_UNKNOWN_SD_CARD;         // SD Card type
unsigned int SD_CardCapacity;                          // SD Card capacity
unsigned char  SD_MaxBusClkFreq = 0;                     // Max. card bus frequency
unsigned char  SD_MID = 0;                               // SD Card Maunufacturer ID
unsigned short SD_OID = 0;                               // SD Card OEM/Application ID


unsigned char  SD_CSD[16];                               // CSD buffer
unsigned char  SD_CID[16];                               // CID buffer
unsigned char  SD_sector[512];                           // sector buffer
unsigned short SD_CRC16_rcv;                             // last received CRC16
unsigned short SD_CRC16_cmp;                             // last computed CRC16


static unsigned char CRC7_one(unsigned char t, unsigned char data) {
	const unsigned char g = 0x89;
	unsigned char i;

	t ^= data;
	for (i = 0; i < 8; i++) {
		if (t & 0x80) t ^= g;
		t <<= 1;
	}

	return t;
}

unsigned char CRC7_buf(const unsigned char * p, unsigned char len) {
	unsigned char j,crc = 0;

	for (j = 0; j < len; j++) crc = CRC7_one(crc,p[j]);

	return crc >> 1;
}

static unsigned short CRC16_one(unsigned short crc, unsigned char ser_data) {
	crc  = (unsigned char)(crc >> 8)|(crc << 8);
	crc ^= ser_data;
	crc ^= (unsigned char)(crc & 0xff) >> 4;
	crc ^= (crc << 8) << 4;
	crc ^= ((crc & 0xff) << 4) << 1;

	return crc;
}

unsigned short CRC16_buf(const unsigned char * p, unsigned int len) {
	unsigned int i;
	unsigned short crc = 0;

	for (i = 0; i < len; i++) crc = CRC16_one(crc,p[i]);

	return crc;
}


unsigned char SD_SendCmd(unsigned char cmd, unsigned int arg) {
	unsigned char wait, response, crc = 0;

	SendAndRcvByte(0xff); // This dummy send necessary for some cards

	// Send: [8b]Command -> [32b]Argument -> [8b]CRC
	SendAndRcvByte(cmd | 0x40);
	crc = CRC7_one(crc,cmd | 0x40);
	SendAndRcvByte(arg >> 24);
	crc = CRC7_one(crc,arg >> 24);
	SendAndRcvByte(arg >> 16);
	crc = CRC7_one(crc,arg >> 16);
	SendAndRcvByte(arg >> 8);
	crc = CRC7_one(crc,arg >> 8);
	SendAndRcvByte(arg);
	crc = CRC7_one(crc,arg);
	SendAndRcvByte(crc | 0x01); // Bit 1 always must be set to "1" in CRC

	// Wait for response from SD Card
	wait = 0;
	while ((response = SendAndRcvByte(0xff)) == 0xff) if (wait++ > 200) break;

	return response;
}

// return values:
//   0xff - SD card timeout
//   0xfe - Unknown or bad SD/MMC card
//   0xfd - Set block size command failed
//   0xfc - bad response for CMD58
//   0xfb - SDv2 pattern mismatch (in response to CMD8)
unsigned char SD_CardInit(void) {
	unsigned int wait = 0, r3;
	unsigned char response;

	MICROSD_CS;

	// Must send at least 74 clock ticks to SD Card
	for (wait = 0; wait < 8; wait++) SendAndRcvByte(0xff);

	// Software SD Card reset
	wait = 0; response = 0x00;
	while (wait < 0x20 && response != 0x01) {
		// Wait for SD card enters idle state (R1 response = 0x01)
		response = SD_SendCmd(SD_CMD_GO_IDLE_STATE,0x00);
		wait++;
	}
	if (wait >= 0x20 && response != 0x01) return 0xff; // SD card timeout

	// CMD8: SEND_IF_COND. Send this command to verify SD card interface operating condition
    /* Argument: - [31:12]: Reserved (shall be set to '0')
	             - [11:8]: Supply Voltage (VHS) 0x1 (Range: 2.7-3.6 V)
	             - [7:0]: Check Pattern (recommended 0xAA) */
	response = SD_SendCmd(SD_CMD_HS_SEND_EXT_CSD,SD_CHECK_PATTERN); // CMD8

	if (response == 0x01) {
		// SDv2 or later
		// Read R7 responce
		r3 =  SendAndRcvByte(0xff) << 24;
		r3 |= SendAndRcvByte(0xff) << 16;
		r3 |= SendAndRcvByte(0xff) << 8;
		r3 |= SendAndRcvByte(0xff);

		if ((r3 & 0x01ff) != (SD_CHECK_PATTERN & 0x01ff)) return 0xfb; // SDv2 pattern mismatch -> unsupported SD card

		// CMD55: Send leading command for ACMD<n> command.
		// CMD41: APP_SEND_OP_COND. For only SDC - initiate initialization process.
		wait = 0; response = 0xff;
		while (++wait < 0x2710 && response != 0x00) {
			SD_SendCmd(SD_CMD_APP_CMD,0); // CMD55
			response = SD_SendCmd(SD_CMD_SD_APP_OP_COND,0x40000000); // ACMD41: HCS flag set
		}
		if (wait >= 0x2710 || response != 0x00) return 0xff; // SD card timeout

		SD_CardType = SD_STD_CAPACITY_SD_CARD_V2_0; // SDv2;

		// Read OCR register
		response = SD_SendCmd(SD_CMD_READ_OCR,0x00000000); // CMD58
		if (response == 0x00) {
			// Get R3 response
			r3  = SendAndRcvByte(0xff) << 24;
			r3 |= SendAndRcvByte(0xff) << 16;
			r3 |= SendAndRcvByte(0xff) << 8;
			r3 |= SendAndRcvByte(0xff);
		} else {
			SD_CardType = SD_UNKNOWN_SD_CARD;
			return 0xfc; // bad CMD58 response
		}
		if (r3 & (1<<30)) SD_CardType = SD_HIGH_CAPACITY_SD_CARD; // SDHC or SDXC
	} else {
		// SDv1 or MMC
		wait = 0; response = 0xff;
		while (++wait < 0xfe) {
			SD_SendCmd(SD_CMD_APP_CMD,0); // CMD55
			response = SD_SendCmd(SD_CMD_SD_APP_OP_COND,0x00000000); // CMD41
			if (response == 0x00) {
				SD_CardType = SD_STD_CAPACITY_SD_CARD_V1_0; // SDv1
				break;
			}
		}

		if (response == 0x05 && wait >= 0xfe) {
			// MMC or bad card
			// CMD1: Initiate initialization process.
			wait = 0; response = 0xff;
			while (++wait < 0xfe) {
				response = SD_SendCmd(SD_CMD_SEND_OP_COND,0x00000000); // CMD1
				if (response == 0x00) {
					SD_CardType = SD_MULTIMEDIA_CARD; // MMC
					break;
				}
			}
		}
	}

	if (SD_CardType == 0) return 0xfe; // Unknown or bad SD/MMC card

	
	// Turn off CRC
	response = SD_SendCmd(SD_CMD_CRC_ON_OFF,0x00000001); // CMD59

	// For SDv2,SDv1,MMC must set block size. For SDHC/SDXC it fixed to 512.
	if ((SD_CardType == SD_STD_CAPACITY_SD_CARD_V1_0) ||
		(SD_CardType == SD_STD_CAPACITY_SD_CARD_V2_0) ||
		(SD_CardType == SD_MULTIMEDIA_CARD))
	{
		response = SD_SendCmd(SD_CMD_SET_BLOCKLEN,0x00000200); // CMD16: block size = 512 bytes
		if (response != 0x00) return 0xfd; // Set block size failed
	}

	MULTI_CS_HIGH;

	return 0;
}

// return:
// 0x00 -- read OK
// 0x01..0xfe -- error response to CMD9
// 0xff -- timeout
unsigned char SD_Read_CSD(void) {
	unsigned int wait;
	unsigned char i, response;
	unsigned int c_size,c_size_mult;
	MICROSD_CS;

	response = SD_SendCmd(SD_CMD_SEND_CSD,0); // CMD9
	if (response != 0x00) {
		// Something wrong happened, fill buffer with zeroes
		for (i = 0; i < 16; i++) SD_CSD[i] = 0x00;
		return response;
	} else {
		wait = 0; response = 0;
		while (++wait <= 0x1ff && response != 0xfe)	response = SendAndRcvByte(0xff);
		if (wait >= 0x1ff) return 0xff;
		// Read 16 bytes of CSD register
		for (i = 0; i < 16; i++) SD_CSD[i] = SendAndRcvByte(0xff);
	}

	// Receive 16-bit CRC (some cards demand this)
	SD_CRC16_rcv  = SendAndRcvByte(0xff) << 8;
	SD_CRC16_rcv |= SendAndRcvByte(0xff);

	// // Calculate CRC16 of received buffer
	SD_CRC16_cmp = CRC16_buf(&SD_CSD[0],16);

	MULTI_CS_HIGH;

	// Parse some stuff from CID
	SD_MaxBusClkFreq = SD_CSD[3];
	
	if (SD_CardType != SD_MULTIMEDIA_CARD) {
		if (SD_CSD[0] >> 6 == 0x01) {
			// CSD Version 2.0
			c_size  = (SD_CSD[7] & 0x3f) << 16;
			c_size |= SD_CSD[8] << 8;
			c_size |= SD_CSD[7];
			SD_CardCapacity = c_size << 9; // = c_size * 512
		} else {
			// CSD Version 1.0
			c_size  = (SD_CSD[6] & 0x03) << 10;
			c_size |= (unsigned int)SD_CSD[7] << 2;
			c_size |= SD_CSD[8] >> 6;
			c_size_mult  = (SD_CSD[9] & 0x03) << 1;
			c_size_mult |= SD_CSD[10] >> 7;
			SD_CardCapacity = c_size << 10;
		}
	} else {
		SD_CardCapacity = 0;
	}

	return 0;
}

// return:
// 0x00 -- read OK
// 0x01..0xfe -- error response to CMD10
// 0xff -- timeout
unsigned char SD_Read_CID(void) {
	unsigned int wait;
	unsigned char i, response;

	MICROSD_CS;

	response = SD_SendCmd(SD_CMD_SEND_CID,0); // CMD10
	if (response != 0x00) {
		// Something wrong happened, fill buffer with zeroes
		for (i = 0; i < 16; i++) SD_CID[i] = 0x00;
		return response;
	} else {
		wait = 0; response = 0;
		while (++wait <= 0x1ff && response != 0xfe)	response = SendAndRcvByte(0xff);
		if (wait >= 0x1ff) return 0xff;
		// Read 16 bytes of CID register
		for (i = 0; i < 16; i++) SD_CID[i] = SendAndRcvByte(0xff);
	}

	// Receive 16-bit CRC (some cards demand this)
	SD_CRC16_rcv  = SendAndRcvByte(0xff) << 8;
	SD_CRC16_rcv |= SendAndRcvByte(0xff);

	// // Calculate CRC16 of received buffer
	SD_CRC16_cmp = CRC16_buf(&SD_CID[0],16);

	MULTI_CS_HIGH;

	return 0;
}

// return:
// 0x00 -- read OK
// 0x01..0xfe -- error response from CMD17
// 0xff -- timeout
unsigned char SD_Read_Block(unsigned int addr) {
	unsigned int wait;
	unsigned short i;
	unsigned char response;

	MICROSD_CS;

	if (SD_CardType != SD_HIGH_CAPACITY_SD_CARD) addr <<= 9; // Convert block number to byte offset
	response = SD_SendCmd(SD_CMD_READ_SINGLE_BLOCK,addr); // CMD17
	if (response != 0x00) {
		// Something wrong happened, fill buffer with zeroes
		for (i = 0; i < 512; i++) SD_sector[i] = 0;
		return response; // SD_CMD_READ_SINGLE_BLOCK command returns bad response
	} else {
		wait = 0; response = 0;
		while (++wait <= 0x1ff && response != 0xfe)	response = SendAndRcvByte(0xff);
		if (wait >= 0x1ff) return 0xff;
		// Read 512 bytes of sector
		for (i = 0; i < 512; i++) SD_sector[i] = SendAndRcvByte(0xff);
	}

	// Receive 16-bit CRC (some cards demand this)
	SD_CRC16_rcv  = SendAndRcvByte(0xff) << 8;
	SD_CRC16_rcv |= SendAndRcvByte(0xff);

	// // Calculate CRC16 of received buffer
	SD_CRC16_cmp = CRC16_buf(&SD_sector[0],512);

	MULTI_CS_HIGH;

	return 0;
}
void SD_TEST (void)
{char buffer[32];
	unsigned char response;
	unsigned char b = 0;
	uart_SendStr("SD_CardInit: ");
	b = SD_CardInit();
	sprintf(buffer, " %d \n\r", b);
		uart_SendBuf(buffer,32);
	
	uart_SendStr("\nCard version: ");
	

	sprintf(buffer, " %d \n\r", SD_CardType);
		uart_SendBuf(buffer,32);
	
	
	response = 0;
	uart_SendStr("\nCSD: ");
	response = SD_Read_CSD();
	if (response != 0x00) {
		uart_SendStr("error = ");
		
	sprintf(buffer, " %x \n\r", response);
		uart_SendBuf(buffer,32);
		
	} else 
	{
		sprintf(buffer, " %x \n\r", (char*)&SD_CSD[0],16);
		uart_SendBuf(buffer,32);
		
	}
	uart_SendStr("\nCID: ");
	response = SD_Read_CID();
	if (response != 0x00) {
		uart_SendStr("error = ");
		sprintf(buffer, " %x \n\r", response);
		uart_SendBuf(buffer,32);
	} else 
	{
		sprintf(buffer, " %x \n\r", (char*)&SD_CID[0],16);
		uart_SendBuf(buffer,32);
		
	}
	uart_SendStr("\nMax bus clk.: ");
	sprintf(buffer, " %d \n\r", SD_MaxBusClkFreq);
		uart_SendBuf(buffer,32);
	
	uart_SendStr("\nDevice size: ");
	
	sprintf(buffer, " %d MB \n\r", (SD_CardCapacity >> 10));
		uart_SendBuf(buffer,32);
	
	//uart_SendStr("Mb\n");

	response = SD_Read_Block(0);
	if (response != 0x00) {
		uart_SendStr("error = ");
		sprintf(buffer, " %x \n\r", response);
		uart_SendBuf(buffer,32);
	} else {
		sprintf(buffer, " %x \n\r", (char*)&SD_sector[0],512,32,'.');
		uart_SendBuf(buffer,32);
		
		uart_SendStr("CRC16: ");
		sprintf(buffer, " %x \n\r", SD_CRC16_rcv);
		uart_SendBuf(buffer,32);
		uart_SendStr(" == ");
		sprintf(buffer, " %x \n\r", SD_CRC16_cmp);
		uart_SendBuf(buffer,32);
	}
	uart_SendByte('\n');

	response = SD_Read_Block(1);
	if (response != 0x00) {
		uart_SendStr("error = ");
		sprintf(buffer, " %x \n\r", response);
		uart_SendBuf(buffer,32);
	} else {
		sprintf(buffer, " %x \n\r", (char*)&SD_sector[0],512,32,'.');
		uart_SendBuf(buffer,32);
		uart_SendStr("CRC16: ");
		sprintf(buffer, " %x \n\r", SD_CRC16_rcv);
		uart_SendBuf(buffer,32);
		uart_SendStr(" == ");
		sprintf(buffer, " %x \n\r", SD_CRC16_cmp);
		uart_SendBuf(buffer,32);
	}
	uart_SendByte('\n');

}