#include "multiplexer.h"
#include "sflash.h"
#include "spi.h"

//M25P128

 /**
   * @brief  Erases the specified FLASH sector.
   * @param  SectorAddr: address of the sector to erase.
   * @retval None
   */
 
	 
 void sFLASH_EraseSector(unsigned long SectorAddr)
 {
  /*!< Send write enable instruction */
   sFLASH_WriteEnable();
 
   /*!< Sector Erase */
   /*!< Select the FLASH: Chip Select low */
   FLASH_CS;
   /*!< Send Sector Erase instruction */
   sFLASH_SendByte(sFLASH_CMD_SE);
   /*!< Send SectorAddr high nibble address byte */
   sFLASH_SendByte((SectorAddr & 0xFF0000) >> 16);
   /*!< Send SectorAddr medium nibble address byte */
   sFLASH_SendByte((SectorAddr & 0xFF00) >> 8);
   /*!< Send SectorAddr low nibble address byte */
   sFLASH_SendByte(SectorAddr & 0xFF);
   /*!< Deselect the FLASH: Chip Select high */
   MULTI_CS_HIGH;
 
   /*!< Wait the end of Flash writing */
   sFLASH_WaitForWriteEnd();
 }
 
 /**
   * @brief  Erases the entire FLASH.
   * @param  None
   * @retval None
   */
 void sFLASH_EraseBulk(void)
 {
   /*!< Send write enable instruction */
  sFLASH_WriteEnable();
 
   /*!< Bulk Erase */
   /*!< Select the FLASH: Chip Select low */
   FLASH_CS;
  /*!< Send Bulk Erase instruction  */
   sFLASH_SendByte(sFLASH_CMD_BE);
   /*!< Deselect the FLASH: Chip Select high */
   MULTI_CS_HIGH;
 
   /*!< Wait the end of Flash writing */
   sFLASH_WaitForWriteEnd();
 }
 
 /**
   * @brief  Writes more than one byte to the FLASH with a single WRITE cycle 
   *         (Page WRITE sequence).
   * @note   The number of byte can't exceed the FLASH page size.
   * @param  pBuffer: pointer to the buffer  containing the data to be written
   *         to the FLASH.
   * @param  WriteAddr: FLASH's internal address to write to.
   * @param  NumByteToWrite: number of bytes to write to the FLASH, must be equal
   *         or less than "sFLASH_PAGESIZE" value.
   * @retval None
   */
 void sFLASH_WritePage(unsigned char* pBuffer, unsigned long WriteAddr, unsigned short NumByteToWrite)
 {
   /*!< Enable the write access to the FLASH */
   sFLASH_WriteEnable();
 
   /*!< Select the FLASH: Chip Select low */
   FLASH_CS;
   /*!< Send "Write to Memory " instruction */
   sFLASH_SendByte(sFLASH_CMD_WRITE);
   /*!< Send WriteAddr high nibble address byte to write to */
   sFLASH_SendByte((WriteAddr & 0xFF0000) >> 16);
   /*!< Send WriteAddr medium nibble address byte to write to */
   sFLASH_SendByte((WriteAddr & 0xFF00) >> 8);
   /*!< Send WriteAddr low nibble address byte to write to */
   sFLASH_SendByte(WriteAddr & 0xFF);
 
   /*!< while there is data to be written on the FLASH */
   while (NumByteToWrite--)
   {
     /*!< Send the current byte */
     sFLASH_SendByte(*pBuffer);
     /*!< Point on the next byte to be written */
     pBuffer++;
   }
 
   /*!< Deselect the FLASH: Chip Select high */
   MULTI_CS_HIGH;
 
   /*!< Wait the end of Flash writing */
   sFLASH_WaitForWriteEnd();
 }
 
 /**
   * @brief  Writes block of data to the FLASH. In this function, the number of
   *         WRITE cycles are reduced, using Page WRITE sequence.
   * @param  pBuffer: pointer to the buffer  containing the data to be written
   *         to the FLASH.
   * @param  WriteAddr: FLASH's internal address to write to.
   * @param  NumByteToWrite: number of bytes to write to the FLASH.
   * @retval None
   */
 void sFLASH_WriteBuffer(unsigned char* pBuffer, unsigned long WriteAddr, unsigned short NumByteToWrite)
 {
   unsigned char NumOfPage = 0, NumOfSingle = 0, Addr = 0, count = 0, temp = 0;
 
   Addr = WriteAddr % sFLASH_SPI_PAGESIZE;
   count = sFLASH_SPI_PAGESIZE - Addr;
   NumOfPage =  NumByteToWrite / sFLASH_SPI_PAGESIZE;
   NumOfSingle = NumByteToWrite % sFLASH_SPI_PAGESIZE;
 
   if (Addr == 0) /*!< WriteAddr is sFLASH_PAGESIZE aligned  */
   {
     if (NumOfPage == 0) /*!< NumByteToWrite < sFLASH_PAGESIZE */
     {
       sFLASH_WritePage(pBuffer, WriteAddr, NumByteToWrite);
     }
     else /*!< NumByteToWrite > sFLASH_PAGESIZE */
     {
       while (NumOfPage--)
       {
         sFLASH_WritePage(pBuffer, WriteAddr, sFLASH_SPI_PAGESIZE);
         WriteAddr +=  sFLASH_SPI_PAGESIZE;
         pBuffer += sFLASH_SPI_PAGESIZE;
       }
 
       sFLASH_WritePage(pBuffer, WriteAddr, NumOfSingle);
     }
   }
   else /*!< WriteAddr is not sFLASH_PAGESIZE aligned  */
   {
     if (NumOfPage == 0) /*!< NumByteToWrite < sFLASH_PAGESIZE */
     {
       if (NumOfSingle > count) /*!< (NumByteToWrite + WriteAddr) > sFLASH_PAGESIZE */
       {
         temp = NumOfSingle - count;
 
         sFLASH_WritePage(pBuffer, WriteAddr, count);
         WriteAddr +=  count;
         pBuffer += count;
 
         sFLASH_WritePage(pBuffer, WriteAddr, temp);
       }
       else
       {
         sFLASH_WritePage(pBuffer, WriteAddr, NumByteToWrite);
       }
     }
     else /*!< NumByteToWrite > sFLASH_PAGESIZE */
     {
       NumByteToWrite -= count;
       NumOfPage =  NumByteToWrite / sFLASH_SPI_PAGESIZE;
       NumOfSingle = NumByteToWrite % sFLASH_SPI_PAGESIZE;
 
       sFLASH_WritePage(pBuffer, WriteAddr, count);
       WriteAddr +=  count;
       pBuffer += count;
 
       while (NumOfPage--)
       {
         sFLASH_WritePage(pBuffer, WriteAddr, sFLASH_SPI_PAGESIZE);
         WriteAddr +=  sFLASH_SPI_PAGESIZE;
         pBuffer += sFLASH_SPI_PAGESIZE;
       }
 
       if (NumOfSingle != 0)
       {
         sFLASH_WritePage(pBuffer, WriteAddr, NumOfSingle);
       }
     }
   }
 }
 
 /**
   * @brief  Reads a block of data from the FLASH.
   * @param  pBuffer: pointer to the buffer that receives the data read from the FLASH.
   * @param  ReadAddr: FLASH's internal address to read from.
   * @param  NumByteToRead: number of bytes to read from the FLASH.
   * @retval None
   */
 void sFLASH_ReadBuffer(unsigned char* pBuffer, unsigned long ReadAddr, unsigned short NumByteToRead)
 {
   /*!< Select the FLASH: Chip Select low */
   FLASH_CS;
 
   /*!< Send "Read from Memory " instruction */
   sFLASH_SendByte(sFLASH_CMD_READ);
 
   /*!< Send ReadAddr high nibble address byte to read from */
   sFLASH_SendByte((ReadAddr & 0xFF0000) >> 16);
   /*!< Send ReadAddr medium nibble address byte to read from */
   sFLASH_SendByte((ReadAddr& 0xFF00) >> 8);
   /*!< Send ReadAddr low nibble address byte to read from */
   sFLASH_SendByte(ReadAddr & 0xFF);
 
   while (NumByteToRead--) /*!< while there is data to be read */
   {
     /*!< Read a byte from the FLASH */
     *pBuffer = sFLASH_SendByte(sFLASH_DUMMY_BYTE);
     /*!< Point to the next location where the byte read will be saved */
     pBuffer++;
   }
 
   /*!< Deselect the FLASH: Chip Select high */
   MULTI_CS_HIGH;
 }
 
 /**
   * @brief  Reads FLASH identification.
   * @param  None
   * @retval FLASH identification
   */
 unsigned long sFLASH_ReadID(void)
 {
   unsigned long Temp = 0, Temp0 = 0, Temp1 = 0, Temp2 = 0;
 
   /*!< Select the FLASH: Chip Select low */
   FLASH_CS;
 
   /*!< Send "RDID " instruction */
   sFLASH_SendByte(0x9F);
 
   /*!< Read a byte from the FLASH */
   Temp0 = sFLASH_SendByte(sFLASH_DUMMY_BYTE);
 
   /*!< Read a byte from the FLASH */
   Temp1 = sFLASH_SendByte(sFLASH_DUMMY_BYTE);
 
   /*!< Read a byte from the FLASH */
   Temp2 = sFLASH_SendByte(sFLASH_DUMMY_BYTE);
 
   /*!< Deselect the FLASH: Chip Select high */
   MULTI_CS_HIGH;
 
   Temp = (Temp0 << 16) | (Temp1 << 8) | Temp2;
 
   return Temp;
 }
 
 /**
   * @brief  Initiates a read data byte (READ) sequence from the Flash.
   *   This is done by driving the /CS line low to select the device, then the READ
   *   instruction is transmitted followed by 3 bytes address. This function exit
   *   and keep the /CS line low, so the Flash still being selected. With this
   *   technique the whole content of the Flash is read with a single READ instruction.
   * @param  ReadAddr: FLASH's internal address to read from.
   * @retval None
   */
 void sFLASH_StartReadSequence(unsigned long ReadAddr)
 {
   /*!< Select the FLASH: Chip Select low */
   FLASH_CS;
 
   /*!< Send "Read from Memory " instruction */
   sFLASH_SendByte(sFLASH_CMD_READ);
 
   /*!< Send the 24-bit address of the address to read from -------------------*/
   /*!< Send ReadAddr high nibble address byte */
   sFLASH_SendByte((ReadAddr & 0xFF0000) >> 16);
   /*!< Send ReadAddr medium nibble address byte */
   sFLASH_SendByte((ReadAddr& 0xFF00) >> 8);
   /*!< Send ReadAddr low nibble address byte */
   sFLASH_SendByte(ReadAddr & 0xFF);
 }
 
 /**
   * @brief  Reads a byte from the SPI Flash.
   * @note   This function must be used only if the Start_Read_Sequence function
   *         has been previously called.
   * @param  None
   * @retval Byte Read from the SPI Flash.
   */
 unsigned char sFLASH_ReadByte(void)
 {
   return (sFLASH_SendByte(sFLASH_DUMMY_BYTE));
 }
 
 /**
   * @brief  Sends a byte through the SPI interface and return the byte received
   *         from the SPI bus.
   * @param  byte: byte to send.
   * @retval The value of the received byte.
   */

 
 /**
   * @brief  Sends a Half Word through the SPI interface and return the Half Word
   *         received from the SPI bus.
   * @param  HalfWord: Half Word to send.
   * @retval The value of the received Half Word.
   */
 /**unsigned short sFLASH_SendHalfWord(unsigned short HalfWord)
 *{
 **/  /*!< Loop while DR register in not emplty */
 /**  while (SPI_I2S_GetFlagStatus(sFLASH_SPI, SPI_I2S_FLAG_TXE) == RESET);
 *
 **/
   /*!< Send Half Word through the sFLASH peripheral */
 /*  SPI_I2S_SendData(sFLASH_SPI, HalfWord);
 */
   /*!< Wait to receive a Half Word */
 //  while (SPI_I2S_GetFlagStatus(sFLASH_SPI, SPI_I2S_FLAG_RXNE) == RESET);
 
   /*!< Return the Half Word read from the SPI bus */
 //  return SPI_I2S_ReceiveData(sFLASH_SPI);
 //}

 /**
   * @brief  Enables the write access to the FLASH.
   * @param  None
   * @retval None
   */
 void sFLASH_WriteEnable(void)
 {
   /*!< Select the FLASH: Chip Select low */
   FLASH_CS;
 
   /*!< Send "Write Enable" instruction */
   sFLASH_SendByte(sFLASH_CMD_WREN);
 
   /*!< Deselect the FLASH: Chip Select high */
   MULTI_CS_HIGH;
 }
 
 /**
   * @brief  Polls the status of the Write In Progress (WIP) flag in the FLASH's
   *         status register and loop until write opertaion has completed.
   * @param  None
   * @retval None
   */
 void sFLASH_WaitForWriteEnd(void)
 {
   unsigned char flashstatus = 0;
 
   /*!< Select the FLASH: Chip Select low */
   FLASH_CS;
 
   /*!< Send "Read Status Register" instruction */
   sFLASH_SendByte(sFLASH_CMD_RDSR);
 
   /*!< Loop as long as the memory is busy with a write cycle */
   do
   {
     /*!< Send a dummy byte to generate the clock needed by the FLASH
     and put the value of the status register in FLASH_Status variable */
     flashstatus = sFLASH_SendByte(sFLASH_DUMMY_BYTE);
 
   }
   while ((flashstatus & sFLASH_WIP_FLAG) == 1); /* Write in progress */
 
   /*!< Deselect the FLASH: Chip Select high */
   MULTI_CS_HIGH;
 }
 void sFLASH_test(void)
 {	unsigned char txbuffer[] = "Hello World!";
	 
	 
	 sFLASH_EraseSector(66);
	 sFLASH_WriteBuffer(txbuffer,66,12);
	 
 }
 /******************* (C) COPYRIGHT 2010 STMicroelectronics *****END OF FILE****/
